@@ -2,7 +2,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Server program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2008 WebIssues Team
+* Copyright (C) 2007-2009 WebIssues Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,7 +10,7 @@
 * (at your option) any later version.
 **************************************************************************/
 
-define( 'VERSION', '0.8.3' );
+define( 'VERSION', '0.8.4' );
 
 if( !include_once( 'config/config.inc.php' ) )
     die( '<p><strong>Fatal Error:</strong> The configuration file <tt>config/config.inc.php</tt> does not exist.</p>' );
@@ -97,6 +97,18 @@ function main()
     if ( !isset( $_SESSION[ 'user_id' ] ) )
         wi_error( 300 );
 
+    if ( $keyword == 'LIST FEATURES' ) {
+        $features = wi_get_features();
+
+        $reply = array();
+        foreach ( $features as $feature ) {
+            $quoted = wi_quote( $feature );
+            $reply[] = "F $quoted";
+        }
+
+        wi_reply( $reply );
+    }
+
     if ( $keyword == 'LIST USERS' ) {
         $query = 'SELECT user_id, user_login, user_name, user_access FROM {users}';
         $rs = wi_query( $query );
@@ -1133,5 +1145,177 @@ function main()
         wi_reply( 'NULL' );
     }
 
+    if ( $keyword == 'LIST PREFERENCES' ) {
+        $user_id = $args[ 1 ];
+
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN && $user_id != $_SESSION[ 'user_id' ] )
+            wi_error( 301 );
+
+        $query = 'SELECT user_id FROM {users} WHERE user_id = %d';
+        if ( !wi_query_row( $query, $user_id ) )
+            wi_error( 314 );
+
+        $query = 'SELECT pref_key, pref_value FROM {preferences} WHERE user_id = %d';
+        $rs = wi_query( $query, $user_id );
+
+        $reply = array();
+        while ( $row = wi_fetch( $rs ) ) {
+            $key = wi_quote( $row[ 'pref_key' ] );
+            $value = wi_quote( $row[ 'pref_value' ] );
+            $reply[] = "P $key $value";
+        }
+
+        wi_reply( $reply );
+    }
+
+    if ( $keyword == 'SET PREFERENCE' ) {
+        $user_id = $args[ 1 ];
+
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN && $user_id != $_SESSION[ 'user_id' ] )
+            wi_error( 301 );
+
+        $query = 'SELECT user_id FROM {users} WHERE user_id = %d';
+        if ( !wi_query_row( $query, $user_id ) )
+            wi_error( 314 );
+
+        $key = wi_string( $args[ 2 ], 40 );
+
+        wi_verify_preference_key( $key );
+
+        $query = 'SELECT pref_value FROM {preferences} WHERE user_id = %d AND pref_key = %s';
+        if ( $row = wi_query_row( $query, $user_id, $key ) )
+            $old_value = $row[ 'pref_value' ];
+        else
+            $old_value = '';
+
+        $new_value = $args[ 3 ];
+
+        $new_value = wi_verify_preference_value( $key, $new_value );
+
+        if ( $new_value == $old_value )
+            wi_reply( 'NULL' );
+
+        if ( $old_value == '' )
+            $query = 'INSERT INTO {preferences} ( user_id, pref_key, pref_value ) VALUES ( %1$d, %2$s, %3$s )';
+        else if ( $new_value == '' )
+            $query = 'DELETE FROM {preferences} WHERE user_id = %1$d AND pref_key = %2$s';
+        else
+            $query = 'UPDATE {preferences} SET pref_value = %3$s WHERE user_id = %1$d AND pref_key = %2$s';
+        wi_query( $query, $user_id, $key, $new_value );
+
+        wi_reply( 'OK' );
+    }
+
+    if ( $keyword == 'LIST NOTIFICATIONS' ) {
+        wi_check_feature( 'notify' );
+
+        $query = 'SELECT f.folder_id'
+            . ' FROM {folders} AS f'
+            . ' JOIN {notify_folders} AS nf ON nf.folder_id = f.folder_id AND nf.user_id = %1$d';
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN )
+            $query .= ' JOIN {rights} AS r ON r.project_id = f.project_id AND r.user_id = %1$d';
+
+        $rs = wi_query( $query, $_SESSION[ 'user_id' ] );
+
+        $reply = array();
+        while ( $row = wi_fetch( $rs ) ) {
+            $folder_id = $row[ 'folder_id' ];
+            $reply[] = "F $folder_id";
+        }
+
+        $query = 'SELECT i.issue_id'
+            . ' FROM {issues} AS i'
+            . ' JOIN {notify_issues} AS ni ON ni.issue_id = i.issue_id AND ni.user_id = %1$d';
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN ) {
+            $query .= ' JOIN {folders} AS f ON f.folder_id = i.folder_id';
+            $query .= ' JOIN {rights} AS r ON r.project_id = f.project_id AND r.user_id = %1$d';
+        }
+
+        $rs = wi_query( $query, $_SESSION[ 'user_id' ] );
+
+        while ( $row = wi_fetch( $rs ) ) {
+            $issue_id = $row[ 'issue_id' ];
+            $reply[] = "I $issue_id";
+        }
+
+        wi_reply( $reply );
+    }
+
+    if ( $keyword == 'NOTIFY FOLDER' ) {
+        wi_check_feature( 'notify' );
+
+        $folder_id = $args[ 1 ];
+
+        $query = 'SELECT f.stamp_id FROM {folders} AS f';
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN )
+            $query .= ' JOIN {rights} AS r ON r.project_id = f.project_id AND r.user_id = %2$d';
+        $query .= ' WHERE f.folder_id = %1$d';
+
+        if ( $row = wi_query_row( $query, $folder_id, $_SESSION[ 'user_id' ] ) )
+            $last_stamp = $row[ 'stamp_id' ];
+        else
+            wi_error( 311 );
+
+        $new_status = $args[ 2 ];
+        if ( $new_status < 0 || $new_status > 1 )
+            wi_error( 342 );
+
+        $query = 'SELECT folder_id FROM {notify_folders} WHERE user_id = %d AND folder_id = %d';
+        if ( $row = wi_query_row( $query, $_SESSION[ 'user_id' ], $folder_id ) )
+            $old_status = 1;
+        else
+            $old_status = 0;
+
+        if ( $old_status == $new_status )
+            wi_reply( 'NULL' );
+
+        if ( $new_status == 1 )
+            $query = 'INSERT INTO {notify_folders} ( user_id, folder_id, stamp_id ) VALUES ( %d, %d, %d )';
+        else
+            $query = 'DELETE FROM {notify_folders} WHERE user_id = %d AND folder_id = %d';
+        wi_query( $query, $_SESSION[ 'user_id' ], $folder_id, $last_stamp );
+
+        wi_reply( 'OK' );
+    }
+
+    if ( $keyword == 'NOTIFY ISSUE' ) {
+        wi_check_feature( 'notify' );
+
+        $issue_id = $args[ 1 ];
+
+        $query = 'SELECT i.stamp_id FROM {issues} AS i';
+        if ( $_SESSION[ 'user_access' ] < ACCESS_ADMIN ) {
+            $query .= ' JOIN {folders} AS f ON f.folder_id = i.folder_id';
+            $query .= ' JOIN {rights} AS r ON r.project_id = f.project_id AND r.user_id = %2$d';
+        }
+        $query .= ' WHERE i.issue_id = %1$d';
+
+        if ( $row = wi_query_row( $query, $issue_id, $_SESSION[ 'user_id' ] ) )
+            $last_stamp = $row[ 'stamp_id' ];
+        else
+            wi_error( 312 );
+
+        $new_status = $args[ 2 ];
+        if ( $new_status < 0 || $new_status > 1 )
+            wi_error( 342 );
+
+        $query = 'SELECT issue_id FROM {notify_issues} WHERE user_id = %d AND issue_id = %d';
+        if ( $row = wi_query_row( $query, $_SESSION[ 'user_id' ], $issue_id ) )
+            $old_status = 1;
+        else
+            $old_status = 0;
+
+        if ( $old_status == $new_status )
+            wi_reply( 'NULL' );
+
+        if ( $new_status == 1 )
+            $query = 'INSERT INTO {notify_issues} ( user_id, issue_id, stamp_id ) VALUES ( %d, %d, %d )';
+        else
+            $query = 'DELETE FROM {notify_issues} WHERE user_id = %d AND issue_id = %d';
+        wi_query( $query, $_SESSION[ 'user_id' ], $issue_id, $last_stamp );
+
+        wi_reply( 'OK' );
+    }
+
     wi_error( 401 );
 }
